@@ -70,6 +70,10 @@ class BaseJobClient(object):
             raise Exception("Unknown Pulsar default file action type %s" % default_file_action)
         self.default_file_action = default_file_action
         self.action_config_path = self.destination_params.get("file_action_config", None)
+        if self.action_config_path is None:
+            self.file_actions = self.destination_params.get("file_actions", {})
+        else:
+            self.file_actions = None
 
         self.setup_handler = build_setup_handler(self, destination_params)
 
@@ -126,7 +130,7 @@ class JobClient(BaseJobClient):
         super(JobClient, self).__init__(destination_params, job_id)
         self.job_manager_interface = job_manager_interface
 
-    def launch(self, command_line, dependencies_description=None, env=[], remote_staging=[], job_config=None):
+    def launch(self, command_line, dependencies_description=None, env=[], remote_staging=[], job_config=None, dynamic_file_sources=None):
         """
         Queue up the execution of the supplied `command_line` on the remote
         server. Called launch for historical reasons, should be renamed to
@@ -157,6 +161,8 @@ class JobClient(BaseJobClient):
             # before queueing.
             setup_params = _setup_params_from_job_config(job_config)
             launch_params['setup_params'] = json_dumps(setup_params)
+        if dynamic_file_sources is not None:
+            launch_params["dynamic_file_sources"] = json_dumps(dynamic_file_sources)
         return self._raw_execute("submit", launch_params)
 
     def full_status(self):
@@ -307,7 +313,7 @@ class BaseMessageJobClient(BaseJobClient):
             raise Exception("full_status() called before a final status was properly cached with cilent manager.")
         return full_status
 
-    def _build_setup_message(self, command_line, dependencies_description, env, remote_staging, job_config):
+    def _build_setup_message(self, command_line, dependencies_description, env, remote_staging, job_config, dynamic_file_sources):
         """
         """
         launch_params = dict(command_line=command_line, job_id=self.job_id)
@@ -321,6 +327,7 @@ class BaseMessageJobClient(BaseJobClient):
         if remote_staging:
             launch_params['remote_staging'] = remote_staging
             launch_params['remote_staging']['ssh_key'] = self.ssh_key
+        launch_params['dynamic_file_sources'] = dynamic_file_sources
         if job_config and self.setup_handler.local:
             # Setup not yet called, job properties were inferred from
             # destination arguments. Hence, must have Pulsar setup job
@@ -341,7 +348,7 @@ class BaseMessageJobClient(BaseJobClient):
 
 class MessageJobClient(BaseMessageJobClient):
 
-    def launch(self, command_line, dependencies_description=None, env=[], remote_staging=[], job_config=None):
+    def launch(self, command_line, dependencies_description=None, env=[], remote_staging=[], job_config=None, dynamic_file_sources=None):
         """
         """
         launch_params = self._build_setup_message(
@@ -350,6 +357,7 @@ class MessageJobClient(BaseMessageJobClient):
             env=env,
             remote_staging=remote_staging,
             job_config=job_config,
+            dynamic_file_sources=dynamic_file_sources,
         )
         response = self.client_manager.exchange.publish("setup", launch_params)
         log.info("Job published to setup message queue: %s", self.job_id)
@@ -372,7 +380,7 @@ class MessageCLIJobClient(BaseMessageJobClient):
         self.remote_pulsar_path = destination_params["remote_pulsar_path"]
         self.shell = shell
 
-    def launch(self, command_line, dependencies_description=None, env=[], remote_staging=[], job_config=None):
+    def launch(self, command_line, dependencies_description=None, env=[], remote_staging=[], job_config=None, dynamic_file_sources=None):
         """
         """
         launch_params = self._build_setup_message(
@@ -381,6 +389,7 @@ class MessageCLIJobClient(BaseMessageJobClient):
             env=env,
             remote_staging=remote_staging,
             job_config=job_config,
+            dynamic_file_sources=dynamic_file_sources,
         )
         base64_message = to_base64_json(launch_params)
         submit_command = os.path.join(self.remote_pulsar_path, "scripts", "submit.bash")
@@ -408,6 +417,7 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
         env=[],
         remote_staging=[],
         job_config=None,
+        dynamic_file_sources=None,
         container_info=None,
         pulsar_app_config=None
     ):
@@ -419,6 +429,7 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
             env=env,
             remote_staging=remote_staging,
             job_config=job_config,
+            dynamic_file_sources=dynamic_file_sources,
         )
         container = None
         guest_ports = None
@@ -468,6 +479,7 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
         params = self.destination_params
 
         pulsar_container_image = self.pulsar_container_image
+        pulsar_container_resources = self._pulsar_container_resources(params)
 
         job_directory = self.job_directory
 
@@ -485,7 +497,10 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
             "workingDir": "/",
             "volumeMounts": volume_mounts,
         }
+        if pulsar_container_resources:
+            pulsar_container_dict["resources"] = pulsar_container_resources
         tool_container_image = container
+        tool_container_resources = self._tool_container_resources(params)
         container_dicts = [pulsar_container_dict]
         if container:
             command = TOOL_EXECUTION_CONTAINER_COMMAND_TEMPLATE % job_directory.job_directory
@@ -497,6 +512,8 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
                 "workingDir": "/",
                 "volumeMounts": volume_mounts,
             }
+            if tool_container_resources:
+                tool_container_spec["resources"] = tool_container_resources
             if guest_ports:
                 tool_container_spec["ports"] = [{"containerPort": int(p)} for p in guest_ports]
             container_dicts.append(tool_container_spec)
@@ -515,6 +532,8 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
             }
         }
         spec = {"template": template}
+        if "k8s_walltime_limit" in params:
+            spec["activeDeadlineSeconds"] = int(params["k8s_walltime_limit"])
         k8s_job_obj = job_object_dict(params, job_name, spec)
         pykube_client = self._pykube_client
         job = Job(pykube_client, k8s_job_obj)
@@ -555,6 +574,26 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
         job_name = produce_unique_k8s_job_name(app_prefix="pulsar", job_id=job_id, instance_id=self.instance_id)
         return job_name
 
+    def _pulsar_container_resources(self, params):
+        return self._container_resources(params, container='pulsar')
+
+    def _tool_container_resources(self, params):
+        return self._container_resources(params, container='tool')
+
+    def _container_resources(self, params, container=None):
+        resources = {}
+        for resource_param in ('requests_cpu', 'requests_memory', 'limits_cpu', 'limits_memory'):
+            subkey, resource = resource_param.split('_', 1)
+            if resource_param in params:
+                if subkey not in resources:
+                    resources[subkey] = {}
+                resources[subkey][resource] = params[resource_param]
+            if container is not None and container + '_' + resource_param in params:
+                if subkey not in resources:
+                    resources[subkey] = {}
+                resources[subkey][resource] = params[container + '_' + resource_param]
+        return resources
+
 
 class InputCachingJobClient(JobClient):
     """
