@@ -2,7 +2,6 @@
 Base Classes and Infrastructure Supporting Concret Manager Implementations.
 
 """
-from collections import deque
 import errno
 import logging
 import os
@@ -16,15 +15,16 @@ from os import getenv
 from os import walk
 import json
 from uuid import uuid4
-import posixpath
 from shutil import rmtree
 
 import six
 
 from pulsar import locks
 from pulsar.managers import ManagerInterface
-from pulsar.client.job_directory import RemoteJobDirectory
-from pulsar.client.job_directory import verify_is_in_directory
+from pulsar.client.job_directory import (
+    RemoteJobDirectory,
+    get_mapped_file,
+)
 
 JOB_DIRECTORY_INPUTS = "inputs"
 JOB_DIRECTORY_OUTPUTS = "outputs"
@@ -170,6 +170,8 @@ class BaseManager(ManagerInterface):
         job_directory = self._job_directory(job_id)
         tool_files_dir = job_directory.tool_files_directory()
         for file in self._list_dir(tool_files_dir):
+            if os.path.isdir(join(tool_files_dir, file)):
+                continue
             contents = open(join(tool_files_dir, file), 'r').read()
             log.debug("job_id: %s - checking tool file %s" % (job_id, file))
             authorization.authorize_tool_file(basename(file), contents)
@@ -240,8 +242,8 @@ class JobDirectory(RemoteJobDirectory):
         """ Verify remote_path is in directory for input_type inputs
         and create directory if needed.
         """
-        directory, allow_nested_files = self._directory_for_file_type(input_type)
-        path = get_mapped_file(directory, remote_path, allow_nested_files=allow_nested_files)
+        directory, allow_nested_files, allow_globs = self._directory_for_file_type(input_type)
+        path = get_mapped_file(directory, remote_path, allow_nested_files=allow_nested_files, allow_globs=allow_globs)
         return path
 
     def read_file(self, name, size=-1, default=None):
@@ -302,6 +304,15 @@ class JobDirectory(RemoteJobDirectory):
         assert self.lock_manager, "Can only use job directory locks if lock manager defined."
         return self.lock_manager.get_lock(self._job_file(name))
 
+    def working_directory_file_contents(self, name):
+        working_directory = self.working_directory()
+        working_directory_path = join(working_directory, name)
+        if exists(working_directory_path):
+            with open(working_directory_path, "rb") as f:
+                return f.read()
+        else:
+            return None
+
     def working_directory_contents(self):
         working_directory = self.working_directory()
         return self.__directory_contents(working_directory)
@@ -361,52 +372,6 @@ class JobDirectory(RemoteJobDirectory):
         self.remove_file(metadata_name)
 
 
-def get_mapped_file(directory, remote_path, allow_nested_files=False, local_path_module=os.path, mkdir=True):
-    """
-
-    >>> import ntpath
-    >>> get_mapped_file(r'C:\\pulsar\\staging\\101', 'dataset_1_files/moo/cow', allow_nested_files=True, local_path_module=ntpath, mkdir=False)
-    'C:\\\\pulsar\\\\staging\\\\101\\\\dataset_1_files\\\\moo\\\\cow'
-    >>> get_mapped_file(r'C:\\pulsar\\staging\\101', 'dataset_1_files/moo/cow', allow_nested_files=False, local_path_module=ntpath)
-    'C:\\\\pulsar\\\\staging\\\\101\\\\cow'
-    >>> get_mapped_file(r'C:\\pulsar\\staging\\101', '../cow', allow_nested_files=True, local_path_module=ntpath, mkdir=False)
-    Traceback (most recent call last):
-    Exception: Attempt to read or write file outside an authorized directory.
-    """
-    if not allow_nested_files:
-        name = local_path_module.basename(remote_path)
-        path = local_path_module.join(directory, name)
-    else:
-        local_rel_path = __posix_to_local_path(remote_path, local_path_module=local_path_module)
-        local_path = local_path_module.join(directory, local_rel_path)
-        verify_is_in_directory(local_path, directory, local_path_module=local_path_module)
-        local_directory = local_path_module.dirname(local_path)
-        if mkdir and not local_path_module.exists(local_directory):
-            os.makedirs(local_directory)
-        path = local_path
-    return path
-
-
-def __posix_to_local_path(path, local_path_module=os.path):
-    """
-    Converts a posix path (coming from Galaxy), to a local path (be it posix or Windows).
-
-    >>> import ntpath
-    >>> __posix_to_local_path('dataset_1_files/moo/cow', local_path_module=ntpath)
-    'dataset_1_files\\\\moo\\\\cow'
-    >>> import posixpath
-    >>> __posix_to_local_path('dataset_1_files/moo/cow', local_path_module=posixpath)
-    'dataset_1_files/moo/cow'
-    """
-    partial_path = deque()
-    while True:
-        if not path or path == '/':
-            break
-        (path, base) = posixpath.split(path)
-        partial_path.appendleft(base)
-    return local_path_module.join(*partial_path)
-
-
 class DirectoryMaker(object):
 
     def __init__(self, mode=None):
